@@ -2,7 +2,7 @@
 #
 #       config.py
 #
-#       Copyright 2009-2011 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2012 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -62,6 +62,7 @@ def config_file_load(inst):
         else: inst.node_path = None
         inst.hpaned_pos = int( dom_iter.attributes["hpaned_pos"].value ) if dom_iter.hasAttribute("hpaned_pos") else 170
         inst.syntax_highlighting = dom_iter.attributes["syntax_highlighting"].value if dom_iter.hasAttribute("syntax_highlighting") else cons.CUSTOM_COLORS_ID
+        inst.style_scheme = dom_iter.attributes["style_scheme"].value if dom_iter.hasAttribute("style_scheme") else cons.STYLE_SCHEME_DEFAULT
         inst.text_font = dom_iter.attributes["text_font"].value if dom_iter.hasAttribute("text_font") else "Sans 9" # default text font
         inst.tree_font = dom_iter.attributes["tree_font"].value if dom_iter.hasAttribute("tree_font") else "Sans 8" # default tree font
         inst.code_font = dom_iter.attributes["code_font"].value if dom_iter.hasAttribute("code_font") else "Monospace 9" # default code font
@@ -118,6 +119,8 @@ def config_file_load(inst):
         inst.backup_copy = (dom_iter.attributes["backup_copy"].value == "True") if dom_iter.hasAttribute("backup_copy") else True
         inst.autosave_on_quit = (dom_iter.attributes["autosave_on_quit"].value == "True") if dom_iter.hasAttribute("autosave_on_quit") else False
         inst.tree_right_side = (dom_iter.attributes["tree_right_side"].value == "True") if dom_iter.hasAttribute("tree_right_side") else False
+        inst.show_white_spaces = (dom_iter.attributes["show_white_spaces"].value == "True") if dom_iter.hasAttribute("show_white_spaces") else True
+        inst.highl_curr_line = (dom_iter.attributes["highl_curr_line"].value == "True") if dom_iter.hasAttribute("highl_curr_line") else True
         inst.nodes_icons = dom_iter.attributes["nodes_icons"].value if dom_iter.hasAttribute("nodes_icons") else "c"
         inst.recent_docs = []
         if dom_iter.hasAttribute("recent_docs"):
@@ -125,11 +128,11 @@ def config_file_load(inst):
             for element in temp_recent_docs:
                 if element: inst.recent_docs.append(base64.b64decode(element))
     else:
-        if sys.platform[0:3] != "win": subprocess.call(cons.SHOW_MENU_ICONS, shell=True)
         inst.file_dir = ""
         inst.file_name = ""
         inst.node_path = None
         inst.syntax_highlighting = cons.CUSTOM_COLORS_ID
+        inst.style_scheme = cons.STYLE_SCHEME_DEFAULT
         inst.tree_font = "Sans 8" # default tree font
         inst.text_font = "Sans 9" # default text font
         inst.code_font = "Monospace 9" # default code font
@@ -164,6 +167,8 @@ def config_file_load(inst):
         inst.backup_copy = True
         inst.autosave_on_quit = False
         inst.tree_right_side = False
+        inst.show_white_spaces = True
+        inst.highl_curr_line = True
         inst.hpaned_pos = 170
         inst.show_node_name_label = True
         inst.nodes_icons = "c"
@@ -201,6 +206,8 @@ def config_file_apply(inst):
     inst.glade.checkbutton_backup_before_saving.set_active(inst.backup_copy)
     inst.glade.checkbutton_autosave_on_quit.set_active(inst.autosave_on_quit)
     inst.glade.checkbutton_tree_right_side.set_active(inst.tree_right_side)
+    inst.glade.checkbutton_show_white_spaces.set_active(inst.show_white_spaces)
+    inst.glade.checkbutton_highlight_current_line.set_active(inst.highl_curr_line)
     inst.glade.checkbutton_start_on_systray.set_active(inst.start_on_systray)
     inst.glade.checkbutton_start_on_systray.set_sensitive(inst.systray)
     # custom link clicked actions
@@ -262,6 +269,7 @@ def config_file_save(inst):
         config.setAttribute("node_path", " ".join(path_list_of_str) )
         config.setAttribute("cursor_position", str(inst.curr_buffer.get_property('cursor-position') ) )
     config.setAttribute("syntax_highlighting", inst.syntax_highlighting)
+    config.setAttribute("style_scheme", inst.style_scheme)
     config.setAttribute("text_font", inst.text_font)
     config.setAttribute("tree_font", inst.tree_font)
     config.setAttribute("code_font", inst.code_font)
@@ -300,6 +308,8 @@ def config_file_save(inst):
     config.setAttribute("backup_copy", str(inst.backup_copy))
     config.setAttribute("autosave_on_quit", str(inst.autosave_on_quit))
     config.setAttribute("tree_right_side", str(inst.tree_right_side))
+    config.setAttribute("show_white_spaces", str(inst.show_white_spaces))
+    config.setAttribute("highl_curr_line", str(inst.highl_curr_line))
     config.setAttribute("nodes_icons", inst.nodes_icons)
     if inst.recent_docs:
         temp_recent_docs = []
