@@ -2,7 +2,7 @@
 #
 #       cons.py
 #
-#       Copyright 2009-2011 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2012 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
 import os, sys
 
 
-VERSION = "0.23.1"
+VERSION = "0.25"
 APP_NAME = "cherrytree"
 NEWER_VERSION_URL = "http://www.giuspen.com/software/version_cherrytree"
 if sys.platform[0:3] == "win":
@@ -38,14 +38,13 @@ else:
     LANG_PATH = os.path.join(os.path.expanduser('~'), '.config/cherrytree/lang')
     IMG_PATH = os.path.join(os.path.expanduser('~'), '.config/cherrytree/img_tmp.png')
     TMP_FOLDER = '/tmp/ct_tmp/'
-    if os.path.isdir('modules'):
+    if os.path.isfile('modules/cons.py'):
         GLADE_PATH = 'glade/'
         LOCALE_PATH = 'locale/'
     else:
-        GLADE_PATH = '/usr/share/cherrytree/glade/'
-        LOCALE_PATH = '/usr/share/locale/'
+        GLADE_PATH = os.path.join(SHARE_PATH, 'cherrytree/glade/')
+        LOCALE_PATH = os.path.join(SHARE_PATH, 'locale')
 AVAILABLE_LANGS = ['default', 'cs', 'de', 'en', 'es', 'fr', 'it', 'pl', 'ru', 'uk']
-SHOW_MENU_ICONS = "gconftool-2 --set /desktop/gnome/interface/menus_have_icons --type bool 1"
 COLOR_YELLOW = "#ffffffff0000"
 COLOR_WHITE = "#ffffffffffff"
 COLOR_BLACK = "#000000000000"
@@ -53,6 +52,7 @@ BAD_CHARS = "[\x00-\x08\x0b-\x1f]"
 TAG_PROPERTIES = ["weight", "foreground", "background", "style", "underline", "strikethrough",
                   "scale", "justification", "link"]
 CUSTOM_COLORS_ID = "custom-colors"
+STYLE_SCHEME_DEFAULT = "cobalt"
 ANCHOR_CHAR = GLADE_PATH + 'anchor_char.png'
 NODES_ICONS = {0:'Red Cherry', 1:'Blue Cherry', 2:'Orange Cherry', 3:'Cyan Cherry',
                4:'Orange Dark Cherry', 5:'Sherbert Cherry', 6:'Yellow Cherry'}
@@ -154,6 +154,61 @@ NODES_STOCKS = ['Node Bullet', 'Node NoIcon', 'Black Cherry',
 
 HORIZONTAL_RULE = "\n=============================================\n"
 
+TABLE_NODE_CREATE = """CREATE TABLE node (
+node_id INTEGER UNIQUE,
+name TEXT,
+txt TEXT,
+syntax TEXT,
+tags TEXT,
+is_ro INTEGER,
+is_richtxt INTEGER,
+has_codebox INTEGER,
+has_table INTEGER,
+has_image INTEGER,
+level INTEGER
+)"""
+
+TABLE_CODEBOX_CREATE = """CREATE TABLE codebox (
+node_id INTEGER,
+offset INTEGER,
+justification TEXT,
+txt TEXT,
+syntax TEXT,
+width INTEGER,
+height INTEGER,
+is_width_pix INTEGER,
+do_highl_bra INTEGER,
+do_show_linenum INTEGER
+)"""
+
+TABLE_TABLE_CREATE = """CREATE TABLE grid (
+node_id INTEGER,
+offset INTEGER,
+justification TEXT,
+txt TEXT,
+col_min INTEGER,
+col_max INTEGER
+)"""
+
+TABLE_IMAGE_CREATE = """CREATE TABLE image (
+node_id INTEGER,
+offset INTEGER,
+justification TEXT,
+anchor TEXT,
+png BLOB
+)"""
+
+TABLE_CHILDREN_CREATE = """CREATE TABLE children (
+node_id INTEGER UNIQUE,
+father_id INTEGER,
+sequence INTEGER
+)"""
+
+TABLE_BOOKMARK_CREATE = """CREATE TABLE bookmark (
+node_id INTEGER UNIQUE,
+sequence INTEGER
+)"""
+
 UI_INFO = """
 <ui>
     <menubar name='MenuBar'>
@@ -161,8 +216,6 @@ UI_INFO = """
             <menuitem action='NewInstance'/>
             <menuitem action='OpenFile'/>
             <separator/>
-            <menuitem action='CTDProtection'/>
-            <separator/>
             <menuitem action='Save'/>
             <menuitem action='SaveAs'/>
             <separator/>
@@ -198,6 +251,8 @@ UI_INFO = """
             <menuitem action='CopyRow'/>
             <menuitem action='DeleteRow'/>
             <menuitem action='DuplicateRow'/>
+            <menuitem action='MoveRowUp'/>
+            <menuitem action='MoveRowDown'/>
         </menu>
 
         <menu action='FormattingMenu'>
@@ -325,6 +380,8 @@ UI_INFO = """
     </toolbar>
 
     <popup name='SysTrayMenu'>
+        <menuitem action='ShowHideMainWin'/>
+        <separator/>
         <menuitem action='ExitApp'/>
     </popup>
 
@@ -385,7 +442,6 @@ def get_entries(inst):
     # name, stock id, label, accelerator, tooltip, callback
     ( "NewInstance", "New Instance", _("New _Instance"), None, _("Start a New Instance of CherryTree"), inst.file_new),
     ( "OpenFile", "gtk-open", _("_Open File"), "<control>O", _("Open a CherryTree Document"), inst.file_open),
-    ( "CTDProtection", "gtk-dialog-authentication", _("Set _Document Protection"), None, _("Set Protection of the Current Document"), inst.dialog_edit_protection),
     ( "Save", "gtk-save", _("_Save"), "<control>S", _("Save File"), inst.file_save),
     ( "SaveAs", "gtk-save-as", _("Save _As"), "<control><shift>S", _("Save File As"), inst.file_save_as),
     ( "Export2HTML", "To HTML", _("Export To _HTML"), None, _("Export To HTML"), inst.export_to_html),
@@ -419,10 +475,12 @@ def get_entries(inst):
     ( "DownCase", "Case Down", _("_Lower Case of Selection/Word"), "<control>W", _("Lower the Case of the Selection/the Underlying Word"), inst.text_selection_lower_case),
     ( "UpCase", "Case Up", _("_Upper Case of Selection/Word"), "<control><shift>W", _("Upper the Case of the Selection/the Underlying Word"), inst.text_selection_upper_case),
     ( "ToggleCase", "Case Toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
-    ( "DuplicateRow", "gtk-add", _("_Duplicate Row/Selection"), "<control>D", _("Duplicate the Whole Row/a Selection"), inst.text_row_selection_duplicate),
-    ( "DeleteRow", "Delete", _("De_lete Row"), "<control>K", _("Delete the Whole Row"), inst.text_row_delete),
-    ( "CutRow", "Cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Whole Row"), inst.text_row_cut),
-    ( "CopyRow", "Copy", _("_Copy Row"), "<control><shift>C", _("Copy the Whole Row"), inst.text_row_copy),
+    ( "DuplicateRow", "gtk-add", _("_Duplicate Row"), "<control>D", _("Duplicate the Current Row/Selection"), inst.text_row_selection_duplicate),
+    ( "MoveRowUp", "gtk-go-up", _("Move _Up Row"), "<alt>Up", _("Move Up the Current Row/Selected Rows"), inst.text_row_up),
+    ( "MoveRowDown", "gtk-go-down", _("Move _Down Row"), "<alt>Down", _("Move Down the Current Row/Selected Rows"), inst.text_row_down),
+    ( "DeleteRow", "Delete", _("De_lete Row"), "<control>K", _("Delete the Current Row/Selected Rows"), inst.text_row_delete),
+    ( "CutRow", "Cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
+    ( "CopyRow", "Copy", _("_Copy Row"), "<control><shift>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
     ( "GoBack", "gtk-go-back", _("Go _Back"), "<alt>Left", _("Go to the Previous Visited Node"), inst.go_back),
     ( "GoForward", "gtk-go-forward", _("Go _Forward"), "<alt>Right", _("Go to the Next Visited Node"), inst.go_forward),
     ( "Undo", "gtk-undo", _("_Undo"), "<control>Z", _("Undo Last Operation"), inst.requested_step_back),
@@ -453,6 +511,7 @@ def get_entries(inst):
     ( "ReplaceAgain", "Find Replace", _("Replace _Again"), "F6", _("Iterate the Last Replace Operation"), inst.replace_again),
     ( "ShowHideNodeNameHeader", "Node Name Header", _("Show/Hide Node Name _Header"), None, _("Toggle Show/Hide Node Name Header"), inst.toggle_show_hide_node_name_header),
     ( "ShowHideTree", "Tree", _("Show/Hide _Tree"), "F9", _("Toggle Show/Hide Tree"), inst.toggle_show_hide_tree),
+    ( "ShowHideMainWin", "CherryTree", _("Show/Hide _CherryTree"), None, _("Toggle Show/Hide CherryTree"), inst.toggle_show_hide_main_window),
     ( "ShowHideToolbar", "Toolbar", _("Show/Hide Tool_bar"), None, _("Toggle Show/Hide Toolbar"), inst.toggle_show_hide_toolbar),
     ( "IncreaseToolbarIconsSize", "gtk-add", _("_Increase Toolbar Icons Size"), None, _("Increase the Size of the Toolbar Icons"), inst.toolbar_icons_size_increase),
     ( "DecreaseToolbarIconsSize", "gtk-remove", _("_Decrease Toolbar Icons Size"), None, _("Decrease the Size of the Toolbar Icons"), inst.toolbar_icons_size_decrease),
@@ -596,10 +655,12 @@ def get_popup_menu_entries_text(inst):
     ("Case Toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("_Row") , "gtk-edit", None, None),
-    ("Cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Whole Row"), inst.text_row_cut),
-    ("Copy", _("C_opy Row"), "<control><shift>C", _("Copy the Whole Row"), inst.text_row_copy),
-    ("Delete", _("De_lete Row"), "<control>K", _("Delete the Whole Row"), inst.text_row_delete),
-    ("gtk-add", _("_Duplicate Row/Selection"), "<control>D", _("Duplicate the Whole Row/a Selection"), inst.text_row_selection_duplicate),
+    ("Cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
+    ("Copy", _("C_opy Row"), "<control><shift>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
+    ("Delete", _("De_lete Row"), "<control>K", _("Delete the Current Row/Selected Rows"), inst.text_row_delete),
+    ("gtk-add", _("_Duplicate Row"), "<control>D", _("Duplicate the Current Row/Selection"), inst.text_row_selection_duplicate),
+    ("gtk-go-up", _("Move _Up Row"), "<alt>Up", _("Move Up the Current Row/Selected Rows"), inst.text_row_up),
+    ("gtk-go-down", _("Move _Down Row"), "<alt>Down", _("Move Down the Current Row/Selected Rows"), inst.text_row_down),
     ("submenu-end", None, None, None, None),
     ("separator", None, None, None, None),
     ("submenu-start", _("_Search") , "Find", None, None),
@@ -632,10 +693,12 @@ def get_popup_menu_entries_code(inst):
     ("Case Toggle", _("_Toggle Case of Selection/Word"), "<control>G", _("Toggle the Case of the Selection/the Underlying Word"), inst.text_selection_toggle_case),
     ("submenu-end", None, None, None, None),
     ("submenu-start", _("_Row") , "gtk-edit", None, None),
-    ("Cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Whole Row"), inst.text_row_cut),
-    ("Copy", _("C_opy Row"), "<control><shift>C", _("Copy the Whole Row"), inst.text_row_copy),
-    ("Delete", _("De_lete Row"), "<control>K", _("Delete the Whole Row"), inst.text_row_delete),
-    ("gtk-add", _("_Duplicate Row/Selection"), "<control>D", _("Duplicate the Whole Row/a Selection"), inst.text_row_selection_duplicate),
+    ("Cut", _("Cu_t Row"), "<control><shift>X", _("Cut the Current Row/Selected Rows"), inst.text_row_cut),
+    ("Copy", _("C_opy Row"), "<control><shift>C", _("Copy the Current Row/Selected Rows"), inst.text_row_copy),
+    ("Delete", _("De_lete Row"), "<control>K", _("Delete the Current Row/Selected Rows"), inst.text_row_delete),
+    ("gtk-add", _("_Duplicate Row"), "<control>D", _("Duplicate the Current Row/Selection"), inst.text_row_selection_duplicate),
+    ("gtk-go-up", _("Move _Up Row"), "<alt>Up", _("Move Up the Current Row/Selected Rows"), inst.text_row_up),
+    ("gtk-go-down", _("Move _Down Row"), "<alt>Down", _("Move Down the Current Row/Selected Rows"), inst.text_row_down),
     ("submenu-end", None, None, None, None),
     ("separator", None, None, None, None),
     ("submenu-start", _("_Search") , "Find", None, None),
