@@ -2,7 +2,7 @@
 #
 #       findreplace.py
 #
-#       Copyright 2009-2011 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2012 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -314,6 +314,7 @@ class FindReplace:
                     self.dad.set_selection_at_offset_n_delta(match_offsets[0] + num_objs,
                                                              len(replacer_text))
                 self.dad.state_machine.update_state(self.dad.treestore[self.dad.curr_tree_iter][3])
+                self.ctdb_handler.pending_edit_db_node_buff(self.dad.treestore[self.dad.curr_tree_iter][3])
             return True
         else: return False
     
@@ -402,7 +403,7 @@ class FindReplace:
             # not first_fromsel or first_fromsel with first_node already parsed
             self.dad.treeview_safe_set_cursor(node_iter)
             if self.parse_current_node_content(pattern, forward, first_fromsel, all_matches, False): return True # not first_node node
-        node_iter = self.dad.treestore.iter_children(node_iter) # check for childrens
+        node_iter = self.dad.treestore.iter_children(node_iter) # check for children
         if node_iter != None and not forward: node_iter = self.dad.get_tree_iter_last_sibling(node_iter)
         while node_iter != None:
             self.all_matches_first_in_node = True
@@ -432,11 +433,12 @@ class FindReplace:
                 replacer_text = self.dad.glade.replace_entry.get_text().decode("utf-8")
                 text_name = text_name.replace(self.curr_find[1], replacer_text)
                 self.dad.treestore[node_iter][1] = text_name
+                self.ctdb_handler.pending_edit_db_node_prop(self.dad.treestore[node_iter][3])
             if not all_matches:
                 self.dad.treeview_safe_set_cursor(node_iter)
                 self.dad.sourceview.grab_focus()
             return True
-        node_iter = self.dad.treestore.iter_children(node_iter) # check for childrens
+        node_iter = self.dad.treestore.iter_children(node_iter) # check for children
         if node_iter != None and not forward: node_iter = self.dad.get_tree_iter_last_sibling(node_iter)
         while node_iter != None:
             if self.parse_node_name(node_iter, pattern, forward, all_matches):
