@@ -2,7 +2,7 @@
 #
 #       machines.py
 #
-#       Copyright 2009-2011 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2012 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -23,6 +23,23 @@ import gtk, xml.dom.minidom, re, base64, copy, StringIO
 import cons, support, exports
 
 
+def get_blob_buffer_from_pixbuf(pixbuf):
+    """Pixbuf To BLOB Buffer"""
+    io = StringIO.StringIO()
+    pixbuf.save_to_callback(io.write, "png")
+    blob_buffer = buffer(io.getvalue())
+    return blob_buffer
+
+def get_pixbuf_from_png_blob_buffer(blob_buffer):
+    """Encoded Buffer To Pixbuf"""
+    pixbuf_loader = gtk.gdk.pixbuf_loader_new_with_mime_type("image/png")
+    try:
+        pixbuf_loader.write(blob_buffer)
+        pixbuf_loader.close()
+        pixbuf = pixbuf_loader.get_pixbuf()
+    except: pixbuf = None
+    return pixbuf
+
 def get_encoded_buffer_from_pixbuf(pixbuf):
     """Pixbuf To Encoded Buffer"""
     io = StringIO.StringIO()
@@ -30,12 +47,14 @@ def get_encoded_buffer_from_pixbuf(pixbuf):
     encoded_buffer = base64.b64encode(io.getvalue())
     return encoded_buffer
 
-def get_pixbuf_from_encoded_buffer(buffer):
+def get_pixbuf_from_encoded_buffer(encoded_buffer):
     """Encoded Buffer To Pixbuf"""
     pixbuf_loader = gtk.gdk.pixbuf_loader_new_with_mime_type("image/png")
-    pixbuf_loader.write(base64.b64decode(buffer))
-    pixbuf_loader.close()
-    pixbuf = pixbuf_loader.get_pixbuf()
+    try:
+        pixbuf_loader.write(base64.b64decode(encoded_buffer))
+        pixbuf_loader.close()
+        pixbuf = pixbuf_loader.get_pixbuf()
+    except: pixbuf = None
     return pixbuf
 
 def get_pixbuf_from_png_encoded_string(png_encoded_string):
@@ -68,28 +87,35 @@ class XMLHandler:
         textbuffer.set_modified(False)
         return True
 
-    def dom_to_treestore(self, ctd, discard_ids, tree_father=None, reset_nodes_names=True):
+    def reset_nodes_names(self):
+        """Reset Nodes Names"""
+        self.dad.nodes_names_dict = {}
+        bookmarks_menu = self.dad.ui.get_widget("/MenuBar/BookmarksMenu").get_submenu()
+        for menu_item in self.dad.bookmarks_menu_items:
+            bookmarks_menu.remove(menu_item)
+        self.dad.bookmarks_menu_items = []
+
+    def dom_to_treestore(self, ctd, discard_ids, tree_father=None):
         """Parse an XML Cherry Tree Document file and build the Tree"""
         dom = xml.dom.minidom.parseString(ctd)
         cherrytree = dom.firstChild
-        self.dad.bookmarks = []
-        if reset_nodes_names:
-            self.dad.nodes_names_dict = {}
-            bookmarks_menu = self.dad.ui.get_widget("/MenuBar/BookmarksMenu").get_submenu()
-            for menu_item in self.dad.bookmarks_menu_items:
-                bookmarks_menu.remove(menu_item)
-            self.dad.bookmarks_menu_items = []
+        if not discard_ids:
+            self.reset_nodes_names()
+            self.dad.bookmarks = []
         if cherrytree.nodeName != "cherrytree": return False
         else:
             dom_iter = cherrytree.firstChild
+            node_sequence = 0
             while dom_iter!= None:
-                if dom_iter.nodeName == "node": self.append_tree_node(dom_iter, tree_father, discard_ids)
+                if dom_iter.nodeName == "node":
+                    node_sequence += 1
+                    self.append_tree_node(dom_iter, tree_father, discard_ids, node_sequence)
                 elif dom_iter.nodeName == "bookmarks":
                     self.dad.bookmarks = dom_iter.attributes['list'].value.split(",")
                 dom_iter = dom_iter.nextSibling
             return True
 
-    def append_tree_node(self, dom_iter, tree_father, discard_ids):
+    def append_tree_node(self, dom_iter, tree_father, discard_ids, node_sequence):
         """Given the dom_iter node, adds it to the tree"""
         if not discard_ids and dom_iter.hasAttribute('unique_id'):
             unique_id = long(dom_iter.attributes['unique_id'].value)
@@ -104,9 +130,8 @@ class XMLHandler:
             syntax_highlighting = syntax_highlighting.lower().replace("C++", "cpp")
             if syntax_highlighting not in self.dad.available_languages:
                 syntax_highlighting = cons.CUSTOM_COLORS_ID
-        if tree_father == None: node_level = 0
-        else: node_level = self.dad.treestore[tree_father][5] + 1
-        cherry = self.dad.get_node_icon(node_level, syntax_highlighting)
+        node_depth = 0 if not tree_father else self.dad.treestore.iter_depth(tree_father)+1
+        cherry = self.dad.get_node_icon(node_depth, syntax_highlighting)
         curr_buffer = self.dad.buffer_create(syntax_highlighting)
         if syntax_highlighting != cons.CUSTOM_COLORS_ID: curr_buffer.begin_not_undoable_action()
         # loop into rich text, write into the buffer
@@ -129,23 +154,28 @@ class XMLHandler:
                                                             curr_buffer,
                                                             unique_id,
                                                             syntax_highlighting,
-                                                            node_level,
+                                                            node_sequence,
                                                             node_tags,
                                                             readonly])
-        self.dad.nodes_names_dict[self.dad.treestore[tree_iter][3]] = self.dad.treestore[tree_iter][1]
+        self.dad.nodes_names_dict[unique_id] = self.dad.treestore[tree_iter][1]
+        if discard_ids:
+            # we are importing nodes
+            self.dad.ctdb_handler.pending_new_db_node(unique_id)
         # loop for child nodes
+        child_sequence = 0
         while child_dom_iter!= None:
-            if child_dom_iter.nodeName == 'rich_text':
+            if child_dom_iter.nodeName == "node":
+                child_sequence += 1
+                self.append_tree_node(child_dom_iter, tree_iter, discard_ids, child_sequence)
+            elif child_dom_iter.nodeName == 'rich_text':
                 support.dialog_error("Rich text instead of child node??!!", self.dad.window)
                 break
-            elif child_dom_iter.nodeName == "node": self.append_tree_node(child_dom_iter, tree_iter, discard_ids)
             child_dom_iter = child_dom_iter.nextSibling
 
     def codebox_deserialize(self, curr_buffer, dom_node):
         """From the XML codebox text to the SourceBuffer"""
         char_offset = int(dom_node.attributes["char_offset"].value)
         justification = dom_node.attributes["justification"].value if dom_node.hasAttribute("justification") else "left"
-        self.dad.curr_buffer = curr_buffer # the table_insert method will need this
         codebox_dict = {
            'frame_width': int(dom_node.attributes['frame_width'].value),
            'frame_height': int(dom_node.attributes['frame_height'].value),
@@ -157,29 +187,35 @@ class XMLHandler:
         }
         self.dad.codeboxes_handler.codebox_insert(curr_buffer.get_iter_at_offset(char_offset),
                                                   codebox_dict,
-                                                  justification)
+                                                  codebox_justification=justification,
+                                                  text_buffer=curr_buffer)
 
     def table_deserialize(self, curr_buffer, dom_node):
         """From the XML table text to the SourceBuffer"""
         char_offset = int(dom_node.attributes["char_offset"].value)
         if dom_node.hasAttribute("justification"): justification = dom_node.attributes["justification"].value
         else: justification = "left"
-        table = {'matrix': [],
-                 'col_min': int(dom_node.attributes['col_min'].value),
-                 'col_max': int(dom_node.attributes["col_max"].value)}
+        table_dict = {
+            'matrix': [],
+            'col_min': int(dom_node.attributes['col_min'].value),
+            'col_max': int(dom_node.attributes["col_max"].value)
+        }
         child_dom_iter = dom_node.firstChild
         while child_dom_iter != None:
             if child_dom_iter.nodeName == "row":
-                table['matrix'].append([])
+                table_dict['matrix'].append([])
                 nephew_dom_iter = child_dom_iter.firstChild
                 while nephew_dom_iter != None:
                     if nephew_dom_iter.nodeName == "cell":
-                        if nephew_dom_iter.firstChild != None: table['matrix'][-1].append(nephew_dom_iter.firstChild.data)
-                        else: table['matrix'][-1].append("")
+                        if nephew_dom_iter.firstChild != None:
+                            table_dict['matrix'][-1].append(nephew_dom_iter.firstChild.data)
+                        else: table_dict['matrix'][-1].append("")
                     nephew_dom_iter = nephew_dom_iter.nextSibling
             child_dom_iter = child_dom_iter.nextSibling
-        self.dad.curr_buffer = curr_buffer # the table_insert method will need this
-        self.dad.tables_handler.table_insert(curr_buffer.get_iter_at_offset(char_offset), table, justification)
+        self.dad.tables_handler.table_insert(curr_buffer.get_iter_at_offset(char_offset),
+                                             table_dict,
+                                             table_justification=justification,
+                                             text_buffer=curr_buffer)
 
     def image_deserialize(self, curr_buffer, dom_node, version):
         """From the XML embedded image text to the SourceBuffer"""
@@ -192,8 +228,10 @@ class XMLHandler:
         else:
             if version == 2: pixbuf = get_pixbuf_from_encoded_buffer(dom_node.firstChild.data)
             else: pixbuf = get_pixbuf_from_png_encoded_string(dom_node.firstChild.data)
-        self.dad.curr_buffer = curr_buffer # the apply_tag method will need this
-        self.dad.image_insert(curr_buffer.get_iter_at_offset(char_offset), pixbuf, justification)
+        if pixbuf: self.dad.image_insert(curr_buffer.get_iter_at_offset(char_offset),
+                                         pixbuf,
+                                         image_justification=justification,
+                                         text_buffer=curr_buffer)
 
     def rich_text_deserialize(self, curr_buffer, dom_node):
         """From the XML rich text to the SourceBuffer"""
@@ -212,7 +250,7 @@ class XMLHandler:
         """Given a treestore iter returns the CherryTree rich text"""
         if "dom" in dir(self): del self.dom
         self.dom = xml.dom.minidom.Document()
-        self.append_dom_node(node_iter, self.dom, to_disk=False)
+        self.append_dom_node(node_iter, self.dom, to_disk=False, skip_children=True)
         return self.dom.toxml()
 
     def treestore_sel_node_only_to_dom(self, tree_iter):
@@ -278,7 +316,7 @@ class XMLHandler:
             self.rich_text_attributes_update(curr_iter, self.curr_attributes)
             tag_found = curr_iter.forward_to_tag_toggle(None)
             while tag_found:
-                self.rich_text_serialize(dom_iter, start_iter, curr_iter, self.curr_attributes)
+                self.rich_txt_serialize(dom_iter, start_iter, curr_iter, self.curr_attributes)
                 if curr_iter.compare(end_iter) == 0: break
                 else:
                     self.rich_text_attributes_update(curr_iter, self.curr_attributes)
@@ -286,7 +324,7 @@ class XMLHandler:
                     start_iter.set_offset(offset_old)
                     tag_found = curr_iter.forward_to_tag_toggle(None)
                     if curr_iter.get_offset() == offset_old: break
-            else: self.rich_text_serialize(dom_iter, start_iter, curr_iter, self.curr_attributes)
+            else: self.rich_txt_serialize(dom_iter, start_iter, curr_iter, self.curr_attributes)
             # in case of writing to disk it's time to serialize the info about the images
             if to_disk == True:
                 pixbuf_table_codebox_vector = self.dad.state_machine.get_embedded_pixbufs_tables_codeboxes(curr_buffer)
@@ -297,9 +335,9 @@ class XMLHandler:
                     elif element[0] == "codebox": self.codebox_element_to_xml(element[1], dom_iter)
         else:
             # plain text insert
-            self.rich_text_serialize(dom_iter, start_iter, end_iter, self.curr_attributes)
+            self.rich_txt_serialize(dom_iter, start_iter, end_iter, self.curr_attributes)
         if not skip_children:
-            tree_iter = self.dad.treestore.iter_children(tree_iter) # check for childrens
+            tree_iter = self.dad.treestore.iter_children(tree_iter) # check for children
             while tree_iter != None:
                 self.append_dom_node(tree_iter, dom_iter, to_disk)
                 tree_iter = self.dad.treestore.iter_next(tree_iter)
@@ -350,9 +388,10 @@ class XMLHandler:
         else: text_iter = dom.createTextNode("anchor")
         dom_iter.appendChild(text_iter)
 
-    def rich_text_serialize(self, dom_node, start_iter, end_iter, curr_attributes, change_case="n"):
+    def rich_txt_serialize(self, dom_node, start_iter, end_iter, curr_attributes, change_case="n", dom=None):
         """Appends a new SourceBuffer part to the XML rich text"""
-        dom_iter = self.dom.createElement("rich_text")
+        if not dom: dom = self.dom
+        dom_iter = dom.createElement("rich_text")
         for tag_property in cons.TAG_PROPERTIES:
             if curr_attributes[tag_property] != "":
                 dom_iter.setAttribute(tag_property, curr_attributes[tag_property])
@@ -362,7 +401,7 @@ class XMLHandler:
             if change_case == "l": slot_text = slot_text.lower()
             elif change_case == "u": slot_text = slot_text.upper()
             elif change_case == "t": slot_text = slot_text.swapcase()
-        text_iter = self.dom.createTextNode(slot_text)
+        text_iter = dom.createTextNode(slot_text)
         dom_iter.appendChild(text_iter)
 
     def rich_text_attributes_update(self, curr_iter, curr_attributes):
@@ -515,15 +554,15 @@ class StateMachine:
     def table_to_dict(self, anchor):
         """Given an Anchor, Returns the Embedded Table as a dictionary"""
         columns_num = len(anchor.headers)
-        table = {'matrix':[], 'col_min': anchor.table_col_min, 'col_max': anchor.table_col_max}
+        table_dict = {'matrix':[], 'col_min': anchor.table_col_min, 'col_max': anchor.table_col_max}
         tree_iter = anchor.liststore.get_iter_first()
         while tree_iter != None:
             row = []
             for column in range(columns_num): row.append(anchor.liststore[tree_iter][column])
-            table['matrix'].append(row)
+            table_dict['matrix'].append(row)
             tree_iter = anchor.liststore.iter_next(tree_iter)
-        table['matrix'].append(copy.deepcopy(anchor.headers))
-        return table
+        table_dict['matrix'].append(copy.deepcopy(anchor.headers))
+        return table_dict
 
     def codebox_to_dict(self, anchor, for_print):
         """Given an Anchor, Returns the Embedded Codebox as a dictionary"""
@@ -576,14 +615,19 @@ class StateMachine:
         }
         self.dad.codeboxes_handler.codebox_insert(iter_insert,
                                                   codebox_dict,
-                                                  element[2])
+                                                  codebox_justification=element[2],
+                                                  text_buffer=text_buffer)
 
-    def apply_image_justification(self, iter_start, justification):
+    def apply_object_justification(self, iter_start, justification, text_buffer):
         """Apply the Proper Justification to an Image"""
         if justification == None: return
         iter_end = iter_start.copy()
         iter_end.forward_char()
-        self.dad.apply_tag("justification", justification, iter_sel_start=iter_start, iter_sel_end=iter_end)
+        self.dad.apply_tag("justification",
+                           justification,
+                           iter_sel_start=iter_start,
+                           iter_sel_end=iter_end,
+                           text_buffer=text_buffer)
 
     def reset(self):
         """State Machine Reset"""
