@@ -2,7 +2,7 @@
 #
 #       tablez.py
 #
-#       Copyright 2009-2011 Giuseppe Penone <giuspen@gmail.com>
+#       Copyright 2009-2012 Giuseppe Penone <giuspen@gmail.com>
 #
 #       This program is free software; you can redistribute it and/or modify
 #       it under the terms of the GNU General Public License as published by
@@ -94,7 +94,8 @@ class TablesHandler:
                                                   parent=self.dad.window)
             if filepath != None:
                 self.dad.pick_dir = os.path.dirname(filepath)
-                file_descriptor = open(filepath, "r")
+                support.text_file_rm_emptylines(filepath)
+                file_descriptor = open(filepath, 'r')
                 reader = UnicodeReader(file_descriptor)
                 table_matrix = []
                 row = reader.next()
@@ -105,8 +106,9 @@ class TablesHandler:
                 table_matrix.append(table_matrix.pop(0))
                 self.table_insert(iter_insert, {'col_min': 40, 'col_max': 1000, 'matrix': table_matrix})
 
-    def table_insert(self, iter_insert, table=None, table_justification=None):
+    def table_insert(self, iter_insert, table=None, table_justification=None, text_buffer=None):
         """Insert a Table at the Given Iter"""
+        if not text_buffer: text_buffer = self.dad.curr_buffer
         if table != None:
             self.table_columns = len(table['matrix'][0])
             self.table_rows = len(table['matrix']) - 1
@@ -117,7 +119,7 @@ class TablesHandler:
             headers = [_("click me")]*self.table_columns
             table_col_min = self.table_col_min
             table_col_max = self.table_col_max
-        anchor = self.dad.curr_buffer.create_child_anchor(iter_insert)
+        anchor = text_buffer.create_child_anchor(iter_insert)
         anchor.liststore = gtk.ListStore(*(str,)*self.table_columns)
         anchor.treeview = gtk.TreeView(anchor.liststore)
         for element in range(self.table_columns):
@@ -157,7 +159,11 @@ class TablesHandler:
                 for column in range(self.table_columns):
                     anchor.liststore[row_iter][column] = table['matrix'][row][column]
         if table_justification:
-            self.dad.state_machine.apply_image_justification(self.dad.curr_buffer.get_iter_at_child_anchor(anchor), table_justification)
+            text_iter = text_buffer.get_iter_at_child_anchor(anchor)
+            self.dad.state_machine.apply_object_justification(text_iter, table_justification, text_buffer)
+        elif self.dad.user_active:
+            # if I apply a justification, the state is already updated
+            self.dad.state_machine.update_state(self.dad.treestore[self.dad.curr_tree_iter][3])
 
     def table_edit_properties(self, *args):
         """Edit Table Properties"""
@@ -405,7 +411,7 @@ class UnicodeReader:
     A CSV reader which will iterate over lines in the CSV file "f",
     which is encoded in the given encoding.
     """
-    def __init__(self, f, dialect=csv.excel, encoding="raw_unicode_escape", **kwds):
+    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
         f = UTF8Recoder(f, encoding)
         self.reader = csv.reader(f, dialect=dialect, **kwds)
 
@@ -425,7 +431,7 @@ class UnicodeWriter:
     which is encoded in the given encoding.
     """
 
-    def __init__(self, f, dialect=csv.excel, encoding="raw_unicode_escape", **kwds):
+    def __init__(self, f, dialect=csv.excel, encoding="utf-8", **kwds):
         # Redirect output to a queue
         self.queue = cStringIO.StringIO()
         self.writer = csv.writer(self.queue, dialect=dialect, **kwds)
